package com.sample;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.compiler.BPMN2ProcessFactory;
import org.drools.compiler.ProcessBuilderFactory;
import org.drools.event.process.ProcessCompletedEvent;
import org.drools.event.process.ProcessEvent;
import org.drools.event.process.ProcessEventListener;
import org.drools.event.process.ProcessNodeLeftEvent;
import org.drools.event.process.ProcessNodeTriggeredEvent;
import org.drools.event.process.ProcessStartedEvent;
import org.drools.event.process.ProcessVariableChangedEvent;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.marshalling.impl.ProcessMarshallerFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessRuntimeFactory;
import org.jbpm.bpmn2.BPMN2ProcessProviderImpl;
import org.jbpm.marshalling.impl.ProcessMarshallerFactoryServiceImpl;
import org.jbpm.process.builder.ProcessBuilderFactoryServiceImpl;
import org.jbpm.process.instance.ProcessRuntimeFactoryServiceImpl;
import org.jbpm.process.workitem.wsht.CommandBasedWSHumanTaskHandler;

import com.sample.util.MyProcessEventListener;

import bitronix.tm.resource.jdbc.PoolingDataSource;

/**
 * This is a sample file to launch a process.
 */
public class JPAPersistProcessTest {

	public static final void main(String[] args) {
		try {

			// load up the knowledge base

			KnowledgeBase kbase = readKnowledgeBase();
			StatefulKnowledgeSession ksession = createSession(kbase);

			int sessionId = ksession.getId();

			System.out.println("We got the session : " + sessionId);

			KnowledgeRuntimeLogger logger = KnowledgeRuntimeLoggerFactory
					.newFileLogger(ksession, "test");
			ksession.getWorkItemManager().registerWorkItemHandler("Human Task",	new CommandBasedWSHumanTaskHandler(ksession));

			
			//added by shashikant on 01/17/2011
			System.out.println("Add the process event listener");
			ProcessEventListener listener = new MyProcessEventListener(ksession);			
			ksession.addEventListener(listener);

			// start a new process instance
			Map<String, Object> map = new HashMap<String, Object>();
			Sample sample = new Sample();
			sample.setId(0);
			sample.setName("Shashikant");
			map.put("sample", sample);

			ksession.insert(sample);

			/*
			 * for(int x=1; x <= 100; x++) { Sample tsample = new Sample();
			 * tsample.setId(x); tsample.setName("Shashikant");
			 * ksession.insert(tsample); }
			 */

			System.out.println("Start the process");	
			ksession.startProcess("com.sample.bpmn.process", map);

			ksession.fireAllRules();

			logger.close();

//			System.out.println("Remove the process event listner");
//			ksession.removeEventListener(listener);
			

		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	private static KnowledgeBase readKnowledgeBase() throws Exception {
		ProcessBuilderFactory
				.setProcessBuilderFactoryService(new ProcessBuilderFactoryServiceImpl());
		ProcessMarshallerFactory
				.setProcessMarshallerFactoryService(new ProcessMarshallerFactoryServiceImpl());
		ProcessRuntimeFactory
				.setProcessRuntimeFactoryService(new ProcessRuntimeFactoryServiceImpl());
		BPMN2ProcessFactory
				.setBPMN2ProcessProvider(new BPMN2ProcessProviderImpl());
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory
				.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("sample.bpmn"),
				ResourceType.BPMN2);
		kbuilder.add(ResourceFactory.newClassPathResource("sample.drl"),
				ResourceType.DRL);
		return kbuilder.newKnowledgeBase();
	}

	private static StatefulKnowledgeSession createSession(KnowledgeBase kbase) {

		Context ctx = null;
		try {
			ctx = new InitialContext();
		} catch (NamingException e) {
			e.printStackTrace();
		}

		initDS();

		Properties properties = new Properties();
		// properties.put("drools.processInstanceManagerFactory",
		// "org.jbpm.process.instance.impl.DefaultProcessInstanceManagerFactory");
		// properties.put("drools.processSignalManagerFactory",
		// "org.jbpm.process.instance.event.DefaultSignalManagerFactory");

		// for jpa configuration
		properties
				.put("drools.processInstanceManagerFactory",
						"org.jbpm.persistence.processinstance.JPAProcessInstanceManagerFactory");
		properties.put("drools.processSignalManagerFactory",
				"org.jbpm.persistence.processinstance.JPASignalManagerFactory");

		KnowledgeSessionConfiguration config = KnowledgeBaseFactory
				.newKnowledgeSessionConfiguration(properties);

		/*** added for jbpm persistence ***/
		EntityManagerFactory emf = Persistence
				.createEntityManagerFactory("org.jbpm.persistence.jpa");
		Environment env = KnowledgeBaseFactory.newEnvironment();
		env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);

		// create a new knowledge session that uses JPA to store the runtime
		// state
		return JPAKnowledgeService.newStatefulKnowledgeSession(kbase, config,
				env);

		/*** completed for jbpm persistence ***/

		// return kbase.newStatefulKnowledgeSession(config,
		// EnvironmentFactory.newEnvironment());
	}

	private static StatefulKnowledgeSession recreateSession(
			KnowledgeBase kbase, int sessionId) {

		Context ctx = null;
		try {
			ctx = new InitialContext();
		} catch (NamingException e) {
			e.printStackTrace();
		}

		initDS();

		Properties properties = new Properties();
		// properties.put("drools.processInstanceManagerFactory",
		// "org.jbpm.process.instance.impl.DefaultProcessInstanceManagerFactory");
		// properties.put("drools.processSignalManagerFactory",
		// "org.jbpm.process.instance.event.DefaultSignalManagerFactory");

		// for jpa configuration
		properties
				.put("drools.processInstanceManagerFactory",
						"org.jbpm.persistence.processinstance.JPAProcessInstanceManagerFactory");
		properties.put("drools.processSignalManagerFactory",
				"org.jbpm.persistence.processinstance.JPASignalManagerFactory");

		KnowledgeSessionConfiguration config = KnowledgeBaseFactory
				.newKnowledgeSessionConfiguration(properties);

		/*** added for jbpm persistence ***/
		EntityManagerFactory emf = Persistence
				.createEntityManagerFactory("org.jbpm.persistence.jpa");
		Environment env = KnowledgeBaseFactory.newEnvironment();
		env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);

		// create a new knowledge session that uses JPA to store the runtime
		// state
		// return JPAKnowledgeService.newStatefulKnowledgeSession( kbase,
		// config, env );

		// recreate the session from database using the sessionId
		return JPAKnowledgeService.loadStatefulKnowledgeSession(sessionId,
				kbase, config, env);
		/*** completed for jbpm persistence ***/

		// return kbase.newStatefulKnowledgeSession(config,
		// EnvironmentFactory.newEnvironment());
	}

	private static void initDS() {

		PoolingDataSource ds = new PoolingDataSource();
		ds.setUniqueName("jdbc/processInstanceDS");
		ds.setClassName("com.mysql.jdbc.jdbc2.optional.MysqlXADataSource");
		ds.setMaxPoolSize(3);
		ds.setAllowLocalTransactions(true);
		ds.getDriverProperties().put("user", "root");
		ds.getDriverProperties().put("password", "password");
		ds.getDriverProperties().put("URL",
				"jdbc:mysql://localhost:3306/taskdb");
		ds.init();

	}
}